// extract mass and heat fluxes through the outcrops and write to .csv 

functions
{
    calHF
    {
        libs                ("libutilityFunctionObjects.so");
        type                coded;
        enabled             true;
        writeControl        adjustableRunTime;
        writeInterval       #calc "$writeInterval/100"; //$writeInterval;
        name                HFcalculation;
        codeWrite
        #{
            const wordList& cZNs = mesh().cellZones().names(); //cellzone names
            label patchID_right = mesh().boundaryMesh().findPatchID("rightOC"); 
            label patchID_left = mesh().boundaryMesh().findPatchID("leftOC");
            label patchID_sediment = mesh().boundaryMesh().findPatchID("topSediment");
        

            const volScalarField& h = mesh().lookupObject<volScalarField>("enthalpy");
            const volScalarField& rho = mesh().lookupObject<volScalarField>("rho");
            const volVectorField& U = mesh().lookupObject<volVectorField>("U");
            const surfaceScalarField& phi = mesh().lookupObject<surfaceScalarField>("phi");
            const surfaceVectorField& Sf = mesh().Sf();
            const surfaceScalarField& magSf = mesh().magSf();

            // total heat output
            // discharge Outcrop
            scalar E_right_out = 0, E_right_in = 0, area_right_out =1e-30, area_right_in =1e-30, phi_right_in = 0, phi_right_out = 0; 
            forAll(phi.boundaryField()[patchID_right], faceI)
            {
                scalar phi0=phi.boundaryField()[patchID_right][faceI];
                scalar h0=h.boundaryField()[patchID_right][faceI];
                scalar sf0 = magSf.boundaryField()[patchID_right][faceI];
                // ---------
                if(phi0>0){
                    E_right_out += phi0*h0;
                    area_right_out += sf0;
                    phi_right_out += phi0;
                }
                if(phi0<0){
                    E_right_in += phi0*h0;
                    area_right_in += sf0;
                    phi_right_in += phi0;
                }
            }
            scalar E_left_out = 0, E_left_in = 0, area_left_out =1e-30, area_left_in =1e-30, phi_left_in = 0, phi_left_out = 0; 
            forAll(phi.boundaryField()[patchID_left], faceI)
            {
                scalar phi1=phi.boundaryField()[patchID_left][faceI];
                scalar h1=h.boundaryField()[patchID_left][faceI];
                scalar sf1 = magSf.boundaryField()[patchID_left][faceI];
                // ---------
                if(phi1>0){ 
                    E_left_out += phi1*h1;
                    area_left_out += sf1;
                    phi_left_out += phi1;
                }
                if(phi1<0){    
                    E_left_in += phi1*h1; 
                    area_left_in += sf1;
                    phi_left_in += phi1;
                }
            }

            scalar E_sed_out = 0, E_sed_in = 0, area_sed_out =1e-30, area_sed_in =1e-30, phi_sed_in = 0, phi_sed_out = 0; 
            forAll(phi.boundaryField()[patchID_sediment], faceI)
            {
                scalar phi2=phi.boundaryField()[patchID_sediment][faceI];
                scalar h2=h.boundaryField()[patchID_sediment][faceI];
                scalar sf2 = magSf.boundaryField()[patchID_sediment][faceI];
                // ---------
                if(phi2>0){ 
                    E_sed_out += phi2*h2;
                    area_sed_out += sf2;
                    phi_sed_out += phi2;
                }
                //Info << "phi = " << phi0 << endl;
                if(phi2<0){    
                    E_sed_in += phi2*h2;
                    area_sed_in += sf2;
                    phi_sed_in += phi2;
                }
            }
            // Sync sum across processors
            reduce(E_right_out, sumOp<scalar>());
            reduce(area_right_out, sumOp<scalar>());
            reduce(E_right_in, sumOp<scalar>());
            reduce(area_right_in, sumOp<scalar>());
            reduce(E_left_out, sumOp<scalar>());
            reduce(area_left_out, sumOp<scalar>());
            reduce(E_left_in, sumOp<scalar>());
            reduce(area_left_in, sumOp<scalar>());
            reduce(E_sed_out, sumOp<scalar>());
            reduce(area_sed_out, sumOp<scalar>());
            reduce(E_sed_in, sumOp<scalar>());
            reduce(area_sed_in, sumOp<scalar>());
            reduce(phi_right_out, sumOp<scalar>());
            reduce(phi_right_in, sumOp<scalar>());
            reduce(phi_left_out, sumOp<scalar>());
            reduce(phi_left_in, sumOp<scalar>());
            reduce(phi_sed_out, sumOp<scalar>());
            reduce(phi_sed_in, sumOp<scalar>());
            
            if (!Pstream::parRun() || (Pstream::parRun() && (Pstream::myProcNo() == Pstream::masterNo())))
            {
                bool exist = Foam::exists("HF.csv");
                std::ofstream fout("HF.csv",std::ofstream::app);
                if(!exist)
                {
                    fout<<"Time(year)"<<",\tTotal Heat output right (W)"<<",\tTotal Heat flux right, out (W/m2)"<<",\tTotal Heat input right (W)"<<",\tTotal Heat flux right, in (W/m2)"<<",\tTotal Heat output left (W)"<<",\tTotal Heat flux left, out (W/m2)"<<",\tTotal Heat input left (W)"<<",\tTotal Heat flux left, in (W/m2)"<<",\tTotal Heat output sediment (W)"<<",\tTotal Heat flux sediment, out (W/m2)"<<",\tTotal Heat input sediment (W)"<<",\tTotal Heat flux sediment, in (W/m2)";
                    fout<<"\n";
                }
                fout<<mesh().time().value()/31536000.0<<"\t"<<E_right_out<<"\t"<<E_right_out/area_right_out<<"\t"<<E_right_in<<"\t"<<E_right_in/area_right_in<<"\t"<<E_left_out<<"\t"<<E_left_out/area_left_out<<"\t"<<E_left_in<<"\t"<<E_left_in/area_left_in<<"\t"<<E_sed_out<<"\t"<<E_sed_out/area_sed_out<<"\t"<<E_sed_in<<"\t"<<E_sed_in/area_sed_in<<"\n";
                //fout<<mesh().time().value()/31536000.0<<"\t"<<E_dis<<"\t"<<"\n";
                fout.close(); 
            } 
            if (!Pstream::parRun() || (Pstream::parRun() && (Pstream::myProcNo() == Pstream::masterNo())))
            {
                bool exist = Foam::exists("phi.csv");
                std::ofstream fout("phi.csv",std::ofstream::app);
                if(!exist)
                {
                    fout<<"Time(year)"<<",\tPhi right, out (kg/s)"<<",\tPhi right, in (kg/s)"<<",\tPhi left, out (kg/s)"<<",\tPhi left, in (kg/s)"<<",\tPhi sediment, out (kg/s)"<<",\tPhi sediment, in (kg/s)";
                    fout<<"\n";
                }
                fout<<mesh().time().value()/31536000.0<<"\t"<<phi_right_out<<"\t"<<phi_right_in<<"\t"<<phi_left_out<<"\t"<<phi_left_in<<"\t"<<phi_sed_out<<"\t"<<phi_sed_in<<"\n";
                //fout<<mesh().time().value()/31536000.0<<"\t"<<E_dis<<"\t"<<"\n";
                fout.close(); 
            } 
        #};
    }
}
