functions
{
    calHF
    {
        libs                ("libutilityFunctionObjects.so");
        type                coded;
        enabled             true;
        writeControl        adjustableRunTime;
        writeInterval       #calc "$writeInterval/100"; //$writeInterval;
        name                HFcalculation;
        codeWrite
        #{
            const wordList& cZNs = mesh().cellZones().names(); //cellzone names
            label patchID_discharge = mesh().boundaryMesh().findPatchID("discharge"); 
            label patchID_recharge = mesh().boundaryMesh().findPatchID("recharge");
            label patchID_sediment = mesh().boundaryMesh().findPatchID("sediment");
        

            const volScalarField& h = mesh().lookupObject<volScalarField>("enthalpy");
            const volScalarField& rho = mesh().lookupObject<volScalarField>("rho");
            const volVectorField& U = mesh().lookupObject<volVectorField>("U");
            const surfaceScalarField& phi = mesh().lookupObject<surfaceScalarField>("phi");
            const surfaceVectorField& Sf = mesh().Sf();
            const surfaceScalarField& magSf = mesh().magSf();

            // total heat output
            // discharge Outcrop
            scalar E_dis_out = 0, E_dis_in = 0, area_dis_out =1e-30, area_dis_in =1e-30, phi_dis_in = 0, phi_dis_out = 0; 
            forAll(phi.boundaryField()[patchID_discharge], faceI)
            {
                scalar phi0=phi.boundaryField()[patchID_discharge][faceI];
                scalar h0=h.boundaryField()[patchID_discharge][faceI];
                scalar sf0 = magSf.boundaryField()[patchID_discharge][faceI];
                // ---------
                if(phi0>0){
                    E_dis_out += phi0*h0;
                    area_dis_out += sf0;
                    phi_dis_out += phi0;
                }
                if(phi0<0){
                    E_dis_in += phi0*h0;
                    area_dis_in += sf0;
                    phi_dis_in += phi0;
                }
            }
            scalar E_re_out = 0, E_re_in = 0, area_re_out =1e-30, area_re_in =1e-30, phi_re_in = 0, phi_re_out = 0; 
            forAll(phi.boundaryField()[patchID_recharge], faceI)
            {
                scalar phi1=phi.boundaryField()[patchID_recharge][faceI];
                scalar h1=h.boundaryField()[patchID_recharge][faceI];
                scalar sf1 = magSf.boundaryField()[patchID_recharge][faceI];
                // ---------
                if(phi1>0){ 
                    E_re_out += phi1*h1;
                    area_re_out += sf1;
                    phi_re_out += phi1;
                }
                //Info << "phi = " << phi0 << endl;
                if(phi1<0){    
                    E_re_in += phi1*h1; 
                    area_re_in += sf1;
                    phi_re_in += phi1;
                }
            }

            scalar E_sed_out = 0, E_sed_in = 0, area_sed_out =1e-30, area_sed_in =1e-30, phi_sed_in = 0, phi_sed_out = 0; 
            forAll(phi.boundaryField()[patchID_sediment], faceI)
            {
                scalar phi2=phi.boundaryField()[patchID_sediment][faceI];
                scalar h2=h.boundaryField()[patchID_sediment][faceI];
                scalar sf2 = magSf.boundaryField()[patchID_sediment][faceI];
                // ---------
                if(phi2>0){ 
                    E_sed_out += phi2*h2;
                    area_sed_out += sf2;
                    phi_sed_out += phi2;
                }
                //Info << "phi = " << phi0 << endl;
                if(phi2<0){    
                    E_sed_in += phi2*h2;
                    area_sed_in += sf2;
                    phi_sed_in += phi2;
                }
            }
            // Sync sum across processors
            reduce(E_dis_out, sumOp<scalar>());
            reduce(area_dis_out, sumOp<scalar>());
            reduce(E_dis_in, sumOp<scalar>());
            reduce(area_dis_in, sumOp<scalar>());
            reduce(E_re_out, sumOp<scalar>());
            reduce(area_re_out, sumOp<scalar>());
            reduce(E_re_in, sumOp<scalar>());
            reduce(area_re_in, sumOp<scalar>());
            reduce(E_sed_out, sumOp<scalar>());
            reduce(area_sed_out, sumOp<scalar>());
            reduce(E_sed_in, sumOp<scalar>());
            reduce(area_sed_in, sumOp<scalar>());
            reduce(phi_dis_out, sumOp<scalar>());
            reduce(phi_dis_in, sumOp<scalar>());
            reduce(phi_re_out, sumOp<scalar>());
            reduce(phi_re_in, sumOp<scalar>());
            reduce(phi_sed_out, sumOp<scalar>());
            reduce(phi_sed_in, sumOp<scalar>());
            
            if (!Pstream::parRun() || (Pstream::parRun() && (Pstream::myProcNo() == Pstream::masterNo())))
            {
                bool exist = Foam::exists("HF.csv");
                std::ofstream fout("HF.csv",std::ofstream::app);
                if(!exist)
                {
                    fout<<"Time(year)"<<",\tTotal Heat output discharge (W)"<<",\tTotal Heat flux discharge, out (W/m2)"<<",\tTotal Heat input discharge (W)"<<",\tTotal Heat flux discharge, in (W/m2)"<<",\tTotal Heat output recharge (W)"<<",\tTotal Heat flux recharge, out (W/m2)"<<",\tTotal Heat input recharge (W)"<<",\tTotal Heat flux recharge, in (W/m2)"<<",\tTotal Heat output sediment (W)"<<",\tTotal Heat flux sediment, out (W/m2)"<<",\tTotal Heat input sediment (W)"<<",\tTotal Heat flux sediment, in (W/m2)";
                    fout<<"\n";
                }
                fout<<mesh().time().value()/31536000.0<<"\t"<<E_dis_out<<"\t"<<E_dis_out/area_dis_out<<"\t"<<E_dis_in<<"\t"<<E_dis_in/area_dis_in<<"\t"<<E_re_out<<"\t"<<E_re_out/area_re_out<<"\t"<<E_re_in<<"\t"<<E_re_in/area_re_in<<"\t"<<E_sed_out<<"\t"<<E_sed_out/area_sed_out<<"\t"<<E_sed_in<<"\t"<<E_sed_in/area_sed_in<<"\n";
                //fout<<mesh().time().value()/31536000.0<<"\t"<<E_dis<<"\t"<<"\n";
                fout.close(); 
            } 
            if (!Pstream::parRun() || (Pstream::parRun() && (Pstream::myProcNo() == Pstream::masterNo())))
            {
                bool exist = Foam::exists("phi.csv");
                std::ofstream fout("phi.csv",std::ofstream::app);
                if(!exist)
                {
                    fout<<"Time(year)"<<",\tPhi discharge, out (kg/s)"<<",\tPhi discharge, in (kg/s)"<<",\tPhi recharge, out (kg/s)"<<",\tPhi recharge, in (kg/s)"<<",\tPhi sediment, out (kg/s)"<<",\tPhi sediment, in (kg/s)";
                    fout<<"\n";
                }
                fout<<mesh().time().value()/31536000.0<<"\t"<<phi_dis_out<<"\t"<<phi_dis_in<<"\t"<<phi_re_out<<"\t"<<phi_re_in<<"\t"<<phi_sed_out<<"\t"<<phi_sed_in<<"\n";
                //fout<<mesh().time().value()/31536000.0<<"\t"<<E_dis<<"\t"<<"\n";
                fout.close(); 
            } 
        #};
    }
}
